@@ -9,12 +9,12 @@ from concurrent.futures import Future
 from threading import Condition as PythonCondition
 
 from satella.coding.decorators import wraps
-from satella.time import measure
+from satella.time import measure, parse_time_string
 from ..typing import ExceptionList
 from ...exceptions import ResourceLocked, WouldWaitMore
 
 
-def call_in_separate_thread(*t_args, **t_kwargs):
+def call_in_separate_thread(*t_args, delay: float = 0, **t_kwargs):
     """
     Decorator to mark given routine as callable in a separate thread.
 
@@ -23,6 +23,8 @@ def call_in_separate_thread(*t_args, **t_kwargs):
 
     The arguments given here will be passed to thread's constructor, so use like:
 
+    :param delay: seconds to wait before launching function
+
     >>> @call_in_separate_thread(daemon=True)
     >>> def handle_messages():
     >>>     while True:
@@ -40,6 +42,8 @@ def call_in_separate_thread(*t_args, **t_kwargs):
                 def run(self):
                     if not self.future.set_running_or_notify_cancel():
                         return
+                    if delay:
+                        time.sleep(delay)
                     try:
                         res = fun(*args, **kwargs)
                         self.future.set_result(res)
@@ -73,17 +77,19 @@ class Condition(PythonCondition):
         warnings.warn('Use notify_all instead', DeprecationWarning)
         self.notify_all()
 
-    def wait(self, timeout: tp.Optional[float] = None,
+    def wait(self, timeout: tp.Optional[tp.Union[str, float]] = None,
              dont_raise: bool = False) -> None:
         """
         Wait for condition to become true.
 
-        :param timeout: timeout to wait. None is default and means infinity
+        :param timeout: timeout to wait. None is default and means infinity. Can be also a
+            time string.
         :param dont_raise: if True, then WouldWaitMore won't be raised
         :raises ResourceLocked: unable to acquire the underlying lock within specified timeout.
         :raises WouldWaitMore: wait's timeout has expired
         """
         if timeout is not None:
+            timeout = parse_time_string(timeout)
             if timeout < 0:
                 timeout = 0
 
@@ -147,18 +153,27 @@ class SingleStartThread(threading.Thread):
 class BogusTerminableThread:
     """
     A mock object that implements threading interface but does nothing
+
+    :ivar running: bool, if it's running
+    :ivar terminated: bool, if terminated
+    :ivar daemon: bool, if daemon
     """
+    __slots__ = ('running', 'terminated', 'daemon')
 
     def __init__(self):
         self.running = False
         self.terminated = False
         self.daemon = True
 
-    def is_alive(self):
+    def is_alive(self) -> bool:
+        """
+        :return: if this thread is alive
+        """
         return not self.terminated and self.running
 
-    def start(self):
+    def start(self) -> None:
         """
+        Set running to True
         :raises RuntimeError: thread already terminated or already running
         """
         if self.terminated:
@@ -167,10 +182,19 @@ class BogusTerminableThread:
             raise RuntimeError('Thread already running')
         self.running = True
 
-    def terminate(self):
+    def terminate(self) -> None:
+        """
+        Set terminated to True.
+
+        Note that to set running to False you need to invoke
+        :meth:`~satella.coding.concurrent.BogusTerminableThread.join` afterwards.
+        """
         self.terminated = True
 
-    def join(self, timeout=None):
+    def join(self, timeout=None) -> None:
+        """
+        Wait for the pseudo-thread. Sets running to False if thread was terminated.
+        """
         if self.terminated:
             self.running = False
 
@@ -213,9 +237,16 @@ class TerminableThread(threading.Thread):
             swallow it and terminate the thread by calling
             :meth:`~satella.coding.concurrent.TerminableThread.terminate`. Note that the
             subclass check will be done via `isinstance` so you can use the metaclass magic :)
+            Note that SystemExit will be automatically added to list of terminable exceptions.
         """
         super().__init__(*args, **kwargs)
         self._terminating = False  # type: bool
+        if terminate_on is None:
+            terminate_on = (SystemExit, )
+        elif isinstance(terminate_on, tuple):
+            terminate_on = (SystemExit, *terminate_on)
+        else:
+            terminate_on = (SystemExit, terminate_on)
         self._terminate_on = terminate_on
 
     @property
@@ -287,6 +318,35 @@ class TerminableThread(threading.Thread):
         self.terminate().join()
         return False
 
+    def safe_wait_condition(self, condition: Condition, timeout: tp.Union[str, float],
+                            wake_up_each: tp.Union[str, float] = 2) -> None:
+        """
+        Wait for a condition, checking periodically if the thread is being terminated.
+
+        To be invoked only by the thread that's represented by the object!
+
+        :param condition: condition to wait on
+        :param timeout: maximum time to wait in seconds. Can be also a time string
+        :param wake_up_each: amount of seconds to wake up each to check for termination.
+            Can be also a time string.
+        :raises WouldWaitMore: timeout has passed and Condition has not happened
+        :raises SystemExit: thread is terminating
+        """
+        timeout = parse_time_string(timeout)
+        wake_up_each = parse_time_string(wake_up_each)
+        t = 0
+        while t < timeout:
+            if self._terminating:
+                raise SystemExit()
+            ttw = min(timeout-t, wake_up_each)
+            t += ttw
+            try:
+                condition.wait(ttw)
+                return
+            except WouldWaitMore:
+                pass
+        raise WouldWaitMore()
+
     def safe_sleep(self, interval: float, wake_up_each: float = 2) -> None:
         """
         Sleep for interval, waking up each wake_up_each seconds to check if terminating,
@@ -294,6 +354,8 @@ class TerminableThread(threading.Thread):
 
         This will do *the right thing* when passed a negative interval.
 
+        To be invoked only by the thread that's represented by the object!
+
         :param interval: Time to sleep in total
         :param wake_up_each: Amount of seconds to wake up each
         :raises SystemExit: thread is terminating
@@ -352,13 +414,13 @@ class IntervalTerminableThread(TerminableThread, metaclass=ABCMeta):
 
     If executing .loop() takes more than x seconds, on_overrun() will be called.
 
-    :param seconds: time that a single looping through should take. This will
-        include the time spent on calling .loop(), the rest of this time will
-        be spent safe_sleep()ing.
+    :param seconds: time that a single looping through should take in seconds.
+        Can be also a time string. This will include the time spent on calling .loop(), the rest
+        of this time will be spent safe_sleep()ing.
     """
 
-    def __init__(self, seconds: float, *args, **kwargs):
-        self.seconds = seconds
+    def __init__(self, seconds: tp.Union[str, float], *args, **kwargs):
+        self.seconds = parse_time_string(seconds)
         super().__init__(*args, **kwargs)
 
     @abstractmethod
